from sasktran2 import pysasktran2 as skinternal
from .config import ConfigBase
from .geometry import GeometryBase
from .viewinggeometry import ViewingGeometryBase
import abc
import numpy as np


class AtmosphereBase(abc.ABC):
    @abc.abstractmethod
    def internal_atmosphere(self, config: ConfigBase,
                            geometry: GeometryBase,
                            viewing_geometry: ViewingGeometryBase) -> skinternal.Atmosphere:
        pass


class Atmosphere(AtmosphereBase):
    def __init__(self, atmosphere: skinternal.Atmosphere):
        self._atmosphere = atmosphere

    def internal_atmosphere(self, config: ConfigBase,
                            geometry: GeometryBase,
                            viewing_geometry: ViewingGeometryBase) -> skinternal.Atmosphere:
        return self._atmosphere

    @classmethod
    def set_manually(cls, nstokes: int, extinction: np.ndarray, ssa: np.ndarray, legendre: np.ndarray, albedo: np.ndarray,
                     dlegendre: np.ndarray = None):
        nwavel = extinction.shape[0]
        ngrid = extinction.shape[1]
        nlegendre = legendre.shape[-1]

        atmo = skinternal.Atmosphere(nstokes, ngrid, nwavel, nlegendre)

        atmo.set_manual_extinction(extinction)
        atmo.set_manual_ssa(ssa)
        atmo.set_manual_lambertian_albedo(albedo)

        if len(legendre.shape) == 4:
            atmo.set_manual_legendre(legendre.reshape((legendre.shape[0], legendre.shape[1], -1)))
        else:
            atmo.set_manual_legendre(legendre)

        if dlegendre is not None:
            atmo.set_manual_legendre_derivatives(dlegendre)

        return Atmosphere(atmo)
